package org.jetbrains.intellij.sharedIndexes.local

import com.intellij.indexing.shared.cdn.*
import java.io.IOException
import java.nio.file.*
import java.nio.file.attribute.BasicFileAttributes
import java.nio.file.attribute.FileTime

fun addFiles(indexesDir: Path, files: Collection<CdnUploadEntry>) {
  println("Adding ${files.size} items...")

  for (newEntry in files) {
    val targetPath = indexesDir.resolve(newEntry.key.replace("/", indexesDir.fileSystem.separator))
    runCatching { Files.createDirectories(targetPath.parent) }
    println("Updating ${newEntry.key} from $newEntry...")

    val ignore = when (newEntry) {
      is CdnUploadDataEntry -> {
        Files.write(targetPath, newEntry.content)
        Files.setLastModifiedTime(targetPath, FileTime.from(newEntry.timestamp))
      }

      is CdnUploadAnotherEntry -> {
        val source = newEntry.unwrap()
        if (source is CdnUploadDataEntry) {
          Files.write(targetPath, source.content)
        } else {
          require(source is LocalDiskEntry) { "Unexpected upload item: $newEntry" }
          Files.copy(source.path, targetPath, StandardCopyOption.REPLACE_EXISTING)
        }
        Files.setLastModifiedTime(targetPath, FileTime.from(newEntry.timestamp))
      }
    }
  }
}

fun wipeFiles(indexesDir: Path, toRemove: Collection<CdnEntry>) {
  if (toRemove.isEmpty()) return
  println("Removing ${toRemove.size} items...")
  for (entry in toRemove) {
    require(entry is LocalDiskEntry) { "Unexpected entry to remove: $entry" }
    println("  Removing ${entry.key} with size ${formatSize(entry.contentLength)}")
    val file = entry.path

    Files.deleteIfExists(file)

    var dir = file.parent
    while (true) {
      if (dir == null) break
      if (!dir.startsWith(indexesDir)) break
      if (dir == indexesDir) break
      if (!Files.isDirectory(dir)) break
      val hasFiles = runCatching {
        Files.list(dir).use { it.findAny().isPresent }
      }.getOrDefault(false)
      if (hasFiles) break

      Files.delete(dir)
      dir = dir.parent
    }
  }
}
